@@ -2,7 +2,7 @@ package Catalyst::Controller::AutoAssets;
 use strict;
 use warnings;
 
-our $VERSION = 0.20;
+our $VERSION = 0.22;
 
 use Moose;
 use namespace::autoclean;
@@ -86,6 +86,9 @@ sub unknown_asset {
   my ($self,$c,$asset) = @_;
   $asset ||= $c->req->path;
   $c->res->status(404);
+  # Clear any other headers that might have been set, like Etag. We don't
+  # want to allow negative caching
+  $c->res->headers->clear;
   $c->res->header( 'Content-Type' => 'text/plain' );
   $c->res->body( "No such asset '$asset'" );
   $self->release_build_lock;
@@ -102,10 +105,6 @@ __END__
 
 Catalyst::Controller::AutoAssets - Automatic asset serving via sha1-based URLs
 
-=head1 VERSION
-
-version 0.19
-
 =head1 SYNOPSIS
 
 In your controller:
@@ -306,12 +305,43 @@ Whether or not to make the current asset available directly via a static path ('
 current_redirect except the asset is served directly. This is essentially only useful for debug purposes
 as it will make no use of caching.
 
+See also 'use_etags' below.
+
 Defaults to false (0).
 
+=head2 current_response_headers
+
+Extra headers to set in the response for 'current' requests. Cache-Control => 'no-cache' is always set unless
+it is overridden here.
+
+Defaults to empty HashRef {}
+
 =head2 static_alias
 
 Alias to use for static requests if C<allow_static_requests> is enabled. Defaults to 'static'.
 
+=head2 static_response_headers
+
+Extra headers to set in the response for 'static' requests. Cache-Control => 'no-cache' is always set unless
+it is overridden here.
+
+Defaults to empty HashRef {}
+
+=head2 use_etags
+
+Whether or not to set 'Etag' ("Entity Tag") HTTP response headers and check 'If-None-Match' client request headers to return
+HTTP/304 'Not Modified' responses to clients that already have the current version of the requested asset/file.
+This is essentially the same default behavior as Apache.
+
+Etags provide another content-based mechanism (built into HTTP 1.1) for cache validation. This module accomplishes
+even better cache validation than Etags because it avoids the validation request needed to check the current Etag in the first place,
+however, Etag functionality has also been included because it is very useful when enabling and using 'static' paths which
+do not make use of the checksum in the URL. Also, when Etags are present, most browsers will use them even when hitting 
+"F5" to manually reload the page to avoid downloading the content again, so this feature further increases performance
+for the F5 use-case which many users may be in the habit of doing for various legit reasons.
+
+Defaults to false (0).
+
 =head2 minify
 
 Whether or not to attempt to minify content for C<CSS> or C<JS> asset types. This is a purely optional
@@ -434,11 +464,31 @@ Convenience method to generate a set of tags, such as CSS <link> and JS <script>
 into the <head> section of an HTML document. What this returns, if anything, is dependent on the asset
 type.
 
-=head1 BUGS
+=head1 BUGS/TODO
 
-Does not currently work on all Windows platforms because of the file locking code.
+=over
+
+=item Does not currently work on all Windows platforms because of the file locking code.
 This will be refactored/generalized in a later version.
 
+=item Rebuilds assets on every request if they are empty (i.e. no files within the include_dir) FIXME
+
+=item Newly added files within a subdirectory do not trigger a rebuild and cannot be accessed, even directly,
+because it does not change the mtime of the top directory.
+See 'all_dirs' option below for a possible fix for this problem. The other fix would be to always check the
+file system for an exact subfile path, even if it does not exist in the subfile_meta data.
+
+=item Needs an 'mtime_check_mode' option to be able to control how thorough the mtime check on every
+request is. This mainly applies to Directory assets and could be tweaked according to the number of files.
+Possible modes could be 'top_dir' (only check the mtime of the top directory, default for Directory), 
+'all_dirs' (check all sub directories), 'all' (check all include files, default for CSS/JS) and 'none'
+to turn off the real-time mtime checks entirely.
+
+=item Needs a 'require_checksum' option to be able to require a specific asset fingerprint (such as for
+included libs that should always have the same checksum, like the ExtJS 3.4.0 release, for instance)
+
+=back
+
 =head1 SEE ALSO
 
 =over
